@@ -291,6 +291,12 @@ void MusicActionMessage::Handle(){
 			case VIOLIN:
 				this->mt->musicActionsBuffer.push_back(0x04);
 				break;
+			case PIANO:
+				this->mt->musicActionsBuffer.push_back(0x05);
+				break;
+			case SAWTOOTH:
+				this->mt->musicActionsBuffer.push_back(0x06);
+				break;
 		}
 
 		//push 2 bytes for music actions counter
