@@ -127,6 +127,12 @@ void PlaybackThread::Run(){
 					case 4:
 						this->instr = aumiks::ViolinInstrument::New();
 						break;
+					case 5:
+						this->instr = aumiks::PianoInstrument::New();
+						break;
+					case 6:
+						this->instr = aumiks::SawtoothInstrument::New();
+						break;
 				}
 			}
 
