@@ -28,10 +28,12 @@ THE SOFTWARE. */
 
 #include <ting/math.hpp>
 #include <ting/types.hpp>
+#include <ting/Buffer.hpp>
 
 #include "PluckedString.hpp"
 #include "../aumiks.hpp"
 #include "synth.hpp"
+#include "FilterOneZero.hpp"
 
 
 
@@ -231,7 +233,7 @@ bool PluckedStringInstrument::Channel::MixToMixBuf(Array<s32>& mixBuf){
 	}
 
 	for(ting::uint i=0; i < mixBuf.Size();){
-		float s = this->pluckedString.Tick();
+		float s = 2 * this->pluckedString.Tick();
 		s32 smp = s32(0x7fff * s);
 
 		//with distortion
@@ -255,6 +257,87 @@ bool PluckedStringInstrument::Channel::MixToMixBuf(Array<s32>& mixBuf){
 
 
 
+PianoInstrument::PianoInstrument(){
+	float xx[] = {
+		0.3728299,  - 0.2122525,  - 0.0674189,  - 0.2268258,  - 0.0089568,     0.0220279,    0.1111207,  - 0.0125976,  - 0.0085174,    0.0192382,   0.0352035,  - 0.0163387,    0.0064105,    0.0052186,    0.0045427
+	};
+
+	float yy[] = {
+		0.0076323,    0.0842224,    0.2661944,    0.1654580,  - 0.0401976,   - 0.0395773,  - 0.0426222,  - 0.0056013,  - 0.0208229,    0.0668553, - 0.0034432,    0.0243109,    0.0041368,    0.0196354,    0.0073706
+	};
+
+	ting::StaticBuffer<float, sizeof(xx)/sizeof(xx[0])> x, y;
+
+	memcpy(x.Buf(), xx, x.SizeInBytes());
+	memcpy(y.Buf(), yy, y.SizeInBytes());
+
+	for(ting::uint i = 0; i < y.Size(); ++i){
+		x[i] /= 2;
+		y[i] /= 2;
+	}
+
+	this->pianoPulseSample = Sample::GenerateFromFourier(
+			x,
+			y,
+			ting::uint((44100.0f / 25.0f) * 1.0f),
+			25
+		);
+}
+
+
+
+//override
+bool PianoInstrument::Channel::MixToMixBuf(Array<s32>& mixBuf){
+	if(this->lastVolume < 0){
+		this->lastVolume = this->volume;
+	}
+	int startVolume = this->lastVolume;
+	int endVolume = this->volume;
+	this->lastVolume = this->volume;
+
+	bool ret = false;
+
+	switch(this->state){
+		case PLAYING:
+			if(this->stopFlag){
+				this->state = RELEASING;
+				this->curTime = 0.2;
+				this->pluckedString.Mute();
+			}
+			break;
+		case RELEASING:
+			this->curTime -= float(mixBuf.Size()) / (2 * 44100.0f);
+			if(curTime <= 0){
+				this->state = STOPPED;
+				ret = true;
+			}
+			break;
+		default:
+			break;
+	}
+
+	for(ting::uint i=0; i < mixBuf.Size();){
+		float s = this->pluckedString.Tick(this->pulseGen.Tick());
+		s32 smp = s32(0x7fff * s);
+
+		//interpolate volume
+		ting::u8 vol = ting::u8(startVolume + ((endVolume - startVolume) * i) / mixBuf.Size());
+
+		//apply volume
+		smp = (smp * vol) / 0xff;
+
+
+		mixBuf[i] += smp;
+		++i;
+		mixBuf[i] += smp;
+		++i;
+	}//~for
+
+	return ret;
+}
+
+
+
 //override
 bool BowedInstrument::Channel::MixToMixBuf(Array<s32>& mixBuf){
 	if(this->lastVolume < 0){
@@ -314,8 +397,8 @@ bool SampledInstrument::Channel::MixToMixBuf(Array<s32>& mixBuf){
 	do{//while(!quit)
 		quit = true;
 		
-		if(this->stopFlag){
-			this->stopFlag = false;
+		if(this->stopFlag && this->envPhase != RELEASE){
+			//NOTE: stopFlag can be rised more than once, so, do the actions only if phase is not set to RELEASE yet.
 			this->curTime = 0;
 			this->envPhase = RELEASE;
 		}
@@ -472,35 +555,23 @@ Ref<Instrument::Channel> SampledInstrument::CreateChannel(){
 
 
 //static
-Ref<SampledInstrument::Sample> TromboneInstrument::GenerateSample(){
-	Ref<Sample> ret(new Sample());
+Ref<aumiks::Sample> TromboneInstrument::GenerateSample(){
+	float cosines[] = {
+		0.002f, -0.256f, -0.177f, -0.268f, -0.056f, 0.010f, 0.013f, 0.001f
+	};
+
+	float sines[] = {
+		-0.181f, -0.128f, 0.122f, 0.021f, 0.034f, -0.024f, 0.000f, -0.004f
+	};
+
+	ting::StaticBuffer<float, sizeof(cosines) / sizeof(cosines[0])> c, s;
+	ASSERT(c.SizeInBytes() == sizeof(cosines))
+	ASSERT(s.SizeInBytes() == sizeof(sines))
+	memcpy(c.Begin(), cosines, c.SizeInBytes());
+	memcpy(s.Begin(), sines, s.SizeInBytes());
 
 	//sampling rate 44100Hz, sound base freq is 25Hz
-	ret->freq = real(25);
-	ret->data.Init(44100 / ting::uint(ret->freq));//hold one period
-
-	Array<real> &d = ret->data;
-
-	typedef float T_Real;
-
-	float f = D2Pi<float>() * float(ret->freq);
-
-	for(ting::uint i = 0; i < ret->data.Size(); ++i){
-		ASSERT(i < d.Size())
-		float t = float(i) * float(1.0f / 44100.0f);
-		d[i] = real(
-				T_Real(0.002f)  * Cos(f * t)             + T_Real(-0.181f) * Sin(f * t) +
-				T_Real(-0.256f) * Cos(T_Real(2) * f * t) + T_Real(-0.128f) * Sin(T_Real(2) * f * t) +
-				T_Real(-0.177f) * Cos(T_Real(3) * f * t) + T_Real(0.122f)  * Sin(T_Real(3) * f * t) +
-				T_Real(-0.268f) * Cos(T_Real(4) * f * t) + T_Real(0.021f)  * Sin(T_Real(4) * f * t) +
-				T_Real(-0.056f) * Cos(T_Real(5) * f * t) + T_Real(0.034f)  * Sin(T_Real(5) * f * t) +
-				T_Real(0.010f)  * Cos(T_Real(6) * f * t) + T_Real(-0.024f) * Sin(T_Real(6) * f * t) +
-				T_Real(0.013f)  * Cos(T_Real(7) * f * t) + T_Real(0.000f)  * Sin(T_Real(7) * f * t) +
-				T_Real(0.001f)  * Cos(T_Real(8) * f * t) + T_Real(-0.004f) * Sin(T_Real(8) * f * t)
-			);
-	}
-
-	return ret;
+	return Sample::GenerateFromFourier(c, s, 44100 / 25, 25);
 }
 
 
@@ -518,7 +589,7 @@ SampledInstrument::ADSRParams TromboneInstrument::GetADSRParams(){
 
 
 //static
-Ref<SampledInstrument::Sample> ClarinetInstrument::GenerateSample(){
+Ref<aumiks::Sample> ClarinetInstrument::GenerateSample(){
 	Ref<Sample> ret(new Sample());
 
 	//sampling rate 44100Hz, sound base freq is 25Hz
@@ -562,16 +633,10 @@ SampledInstrument::ADSRParams ClarinetInstrument::GetADSRParams(){
 
 
 //static
-Ref<SampledInstrument::Sample> ViolinInstrument::GenerateSample(){
-	Ref<Sample> ret(new Sample());
-
-	//sampling rate 44100Hz, sound base freq is 25Hz
-	ret->freq = real(25);
-	ret->data.Init(44100 / ting::uint(ret->freq));//hold one period
-
+Ref<aumiks::Sample> ViolinInstrument::GenerateSample(){
 	typedef float T_Real;
 
-	T_Real y[] = {
+	T_Real yData[] = {
 		0,
 		0.4,
 		0.28,
@@ -599,7 +664,7 @@ Ref<SampledInstrument::Sample> ViolinInstrument::GenerateSample(){
 		0
 	};
 
-	T_Real x[] = {
+	T_Real xData[] = {
 		0,
 		4,
 		8,
@@ -627,30 +692,20 @@ Ref<SampledInstrument::Sample> ViolinInstrument::GenerateSample(){
 		152
 	};
 
-	ting::uint numPoints = sizeof(x) / sizeof(x[0]);
-	ASSERT(numPoints == sizeof(y) / sizeof(y[0]))
-	ASSERT(numPoints > 0)
-
-	T_Real t = T_Real(ret->data.Size());//period in samples
-	T_Real d = t / x[numPoints - 1];// time scale
-
-	ting::uint j = 0;
-
-	for(ting::uint i = 0; i < numPoints - 1 && j < ret->data.Size(); ++i){
-		T_Real segLenX = (x[i + 1] - x[i]) * d;
-		T_Real segDelta = y[i + 1] - y[i];
-		T_Real k = segDelta/segLenX;
-
-		T_Real b = - k * x[i] * d + y[i];
-
-		while(j < x[i + 1] * d && j < ret->data.Size()){
-			ASSERT_INFO(j < ret->data.Size(), "j = " << j << " ret->data.Size() = " << ret->data.Size())
-			ret->data[j] = k * j + b;
-//			TRACE(<< (ret->data[j]) << std::endl)
-			++j;
-		}
-	}
+	ting::StaticBuffer<float, sizeof(xData) / sizeof(xData[0])> x, y;
+	ASSERT(x.SizeInBytes() == sizeof(x))
+	ASSERT(y.SizeInBytes() == sizeof(y))
+	memcpy(x.Begin(), xData, x.SizeInBytes());
+	memcpy(y.Begin(), yData, y.SizeInBytes());
 
+	//sampling rate 44100Hz, need to hold one period
+	//sound base freq is 25Hz
+	Ref<Sample> ret = Sample::GenerateFromLineString(
+			x,
+			y,
+			44100 / ting::uint(25),
+			25
+		);
 
 	return ret;
 }
@@ -659,11 +714,60 @@ Ref<SampledInstrument::Sample> ViolinInstrument::GenerateSample(){
 
 SampledInstrument::ADSRParams ViolinInstrument::GetADSRParams(){
 	ADSRParams adsr = {
-		30, //attack
+		20, //attack
 		100, //decay
-		0.9, //sustainLevel
+		0.8, //sustainLevel
 		20 //release
 	};
 	return adsr;
 }
 
+
+
+//static
+Ref<aumiks::Sample> SawtoothInstrument::GenerateSample(){
+	typedef float T_Real;
+
+	T_Real yData[] = {
+		0, 1, -1, 0
+	};
+
+	T_Real xData[] = {
+		0, 50, 51, 100
+	};
+
+	ting::StaticBuffer<float, sizeof(xData) / sizeof(xData[0])> x, y;
+	ASSERT(x.SizeInBytes() == sizeof(x))
+	ASSERT(y.SizeInBytes() == sizeof(y))
+	memcpy(x.Begin(), xData, x.SizeInBytes());
+	memcpy(y.Begin(), yData, y.SizeInBytes());
+
+	//sampling rate 44100Hz, need to hold one period
+	//sound base freq is 25Hz
+	Ref<Sample> ret = Sample::GenerateFromLineString(
+			x,
+			y,
+			44100 / ting::uint(25),
+			25
+		);
+
+	FilterOneZero f(0.5, 0.5);
+
+	for(ting::uint i = 0; i < ret->data.Size(); ++i){
+		ret->data[i] = f.Tick(ret->data[i]);
+	}
+
+	return ret;
+}
+
+
+
+SampledInstrument::ADSRParams SawtoothInstrument::GetADSRParams(){
+	ADSRParams adsr = {
+		20, //attack
+		100, //decay
+		0.8, //sustainLevel
+		50 //release
+	};
+	return adsr;
+}
