@@ -39,6 +39,8 @@ THE SOFTWARE. */
 
 #include "PluckedString.hpp"
 #include "Bowed.hpp"
+#include "Sample.hpp"
+#include "SamplePulseGen.hpp"
 
 namespace aumiks{
 
@@ -119,6 +121,66 @@ public:
 
 
 
+class PianoInstrument : public Instrument{
+	ting::Ref<Sample> pianoPulseSample;
+	
+public:
+	class Channel : public Instrument::Channel{
+		SamplePulseGen pulseGen;
+		PluckedString pluckedString;
+
+		enum E_State{
+			STOPPED,
+			PLAYING,
+			RELEASING
+		} state;
+
+		float curTime;
+
+	public:
+		Channel(const ting::Ref<Sample> pulseSample) :
+				pulseGen(pulseSample),
+				pluckedString(27.5),
+				state(STOPPED)
+		{
+			this->SetFrequency(220);
+			this->pluckedString.SetLoopGain(1);
+			this->pluckedString.SetLoopFilterCoefs(0.95, 0.05);
+		}
+
+		//override
+		void SetFrequency(float freq){
+			this->pluckedString.SetFrequency(freq);
+			this->pulseGen.SetFrequency(freq);
+		}
+
+	protected:
+		//override
+		void OnStart(){
+			this->pluckedString.Init();
+			this->pulseGen.Init();
+			this->state = PLAYING;
+		}
+	private:
+		//override
+		bool MixToMixBuf(Array<s32>& mixBuf);
+	};
+
+	PianoInstrument();
+
+	//override
+	Ref<Instrument::Channel> CreateChannel(){
+		Ref<PianoInstrument::Channel> ch(new PianoInstrument::Channel(this->pianoPulseSample));
+		return ch;
+	}
+
+	static inline ting::Ref<PianoInstrument> New(){
+		return ting::Ref<PianoInstrument>(new PianoInstrument());
+	}
+};
+
+
+
 class BowedInstrument : public Instrument{
 public:
 	class Channel : public Instrument::Channel{
@@ -266,7 +328,7 @@ public:
 
 		Channel(Ref<SampledInstrument> instrument) :
 				instr(instrument),
-				curPhase(0),
+				curPhase(rand() % instrument->sample->data.Size()),
 				curTime(0),
 				envPhase(ATTACK)
 		{
@@ -296,12 +358,6 @@ public:
 
 
 protected:
-	class Sample : public RefCounted{
-	public:
-		Array<real> data;//sampling rate 44100Hz
-		real freq;//base frequency of the sound
-	};
-
 	SampledInstrument(
 			const Ref<Sample> sampleData,
 			ADSRParams adsrParameters
@@ -311,7 +367,7 @@ protected:
 	{}
 
 private:
-	const Ref<Sample> sample;
+	const Ref<aumiks::Sample> sample;
 
 	ADSRParams adsrParams;
 };
@@ -372,4 +428,22 @@ public:
 
 
 
+class SawtoothInstrument : public SampledInstrument{
+	SawtoothInstrument() :
+			SampledInstrument(GenerateSample(), GetADSRParams())
+	{}
+
+
+	static Ref<Sample> GenerateSample();
+
+	static ADSRParams GetADSRParams();
+
+public:
+	static inline ting::Ref<SawtoothInstrument> New(){
+		return ting::Ref<SawtoothInstrument>(new SawtoothInstrument());
+	}
+};
+
+
+
 }//~namespace
