@@ -56,6 +56,8 @@ enum E_Timbre{
 	TROMBONE,
 	CLARINET,
 	VIOLIN,
+	PIANO,
+	SAWTOOTH,
 
 	NUM_TIMBRES
 };
